<?php

namespace Controller;

use Exception\CustomException;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Util\Messages;
use Util\Utilities;

class DownloadVersion extends Simple
{
    /** @var  \Service\DownloadVersion */
    protected $downloadVersionService;

    function __construct(\Service\DownloadVersion $downloadVersionService) {
        $this->downloadVersionService = $downloadVersionService;
    }

    public function getProductsAvailablesByVersion(Request\Filter $request, Response $response) {
        try {
            $filter            = $request->getFilterCriteria()->getConditions();
            $version           = Utilities::getValueInFilterDataSourceByName($filter, 'VERSION', true);
            $versionPatch      = Utilities::getValueInFilterDataSourceByName($filter, 'VERSION_PATCH', true);
            $languageId        = Utilities::getValueInFilterDataSourceByName($filter, 'LANGUAGE_ID', true);
            $productsAvailable = $this->downloadVersionService->getProductsAvailablesByVersion($version, $versionPatch, $languageId);
            $response->addDataSet(new DataSet('findProductsAvailablesByVersion', $productsAvailable));
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }
    
    public function getDescriptionByRequest(Request\Filter $request, Response $response) {
        try {
            $filter               = $request->getFilterCriteria()->getConditions();
            $requestId            = Utilities::getValueInFilterDataSourceByName($filter, 'REQUEST_ID', true);
            $descriptionByRequest = $this->downloadVersionService->getDescriptionByRequest($requestId);
            $response->addDataSet(new DataSet('descriptionByRequest', $descriptionByRequest));
        } catch(\Exception $ex) {
            $response->addMessage(new Response\Message($ex->getMessage()));
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function getRequestsAnswered(Request\Filter $request, Response $response) {
        try {
            $filter           = $request->getFilterCriteria()->getConditions();
            $initialPatch     = Utilities::getValueInFilterDataSourceByName($filter, 'CURRENT_PATCH', true);
            $endPatch         = Utilities::getValueInFilterDataSourceByName($filter, 'LAST_PATCH', true);
            $currentClient    = Utilities::getValueInFilterDataSourceByName($filter, 'CURRENT_CLIENT', true);
            $requestsAnswered = $this->downloadVersionService->getRequestsAnswered($currentClient, $initialPatch, $endPatch, $filter);
            foreach ($requestsAnswered as &$request) {
                $request['REQUEST_DESCRIPTION'] = Utilities::rtf2textPlain($request['REQUEST_DESCRIPTION']);
                $request['DSSOLISERV'] = Utilities::rtf2textPlain($request['DSSOLISERV']);
            }
            $response->addDataSet(new DataSet('findRequestsAnswered', $requestsAnswered));
        } catch(\Exception $ex) {
            $response->addMessage(new Response\Message($ex->getMessage()));
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function getProductVersionsByPatchesInterval(Request\Filter $request, Response $response)
    {
        try {
            $filter       = $request->getFilterCriteria()->getConditions();
            $initialPatch = Utilities::getValueInFilterDataSourceByName($filter, 'INITIAL_PATCH', true);
            $endPatch     = Utilities::getValueInFilterDataSourceByName($filter, 'LAST_PATCH', true);
            $productVersionsByPatchesInterval = $this->downloadVersionService->getProductVersionsByPatchesInterval($initialPatch, $endPatch);
            $response->addDataSet(new DataSet('findProductVersionsByPatchesInterval', $productVersionsByPatchesInterval));
        } catch(\Exception $ex) {
            $response->addMessage(new Response\Message($ex->getMessage()));
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function getReportChangedFiles(Request\Filter $request, Response $response) {
        try {
            $filter       = $request->getFilterCriteria()->getConditions();
            $initialPatch = Utilities::getValueInFilterDataSourceByName($filter, 'INITIAL_PATCH', true);
            $endPatch     = Utilities::getValueInFilterDataSourceByName($filter, 'LAST_PATCH', true);
            $report       = $this->downloadVersionService->getReportChangedFiles($initialPatch, $endPatch);
            $response->addDataSet(new DataSet('reportChangedFiles', array($report)));
        } catch(\Exception $ex) {
            $response->addMessage(new Response\Message($ex->getMessage()));
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function getAuthorizationByCurrentUser(Request\Filter $request, Response $response) {
        try {
            $response->addDataSet(new DataSet('authorizationByCurrentUser', array($this->downloadVersionService->getAuthorizationByCurrentUser())));
        } catch(\Exception $ex) {
            $response->addMessage(new Response\Message($ex->getMessage()));
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function requestPatchGeneration(Request\DataSet $request, Response $response) {
        try {
            $rows           = $request->getDataSet()->getRows()[0];
            $email          = isset($rows['EMAIL']) ? $rows['EMAIL'] : null;
            $version        = isset($rows['VERSION']) ? $rows['VERSION'] : null;
            $initialPatch   = isset($rows['INITIAL_PATCH']) ? $rows['INITIAL_PATCH'] : null;
            $lastPatch      = isset($rows['LAST_PATCH']) ? $rows['LAST_PATCH'] : null;
            $productId      = isset($rows['PRODUCT_ID']) ? $rows['PRODUCT_ID'] : null;
            $requestType    = isset($rows['REQUEST_TYPE']) ? $rows['REQUEST_TYPE'] : null;
            $requestNumber  = isset($rows['REQUEST_NUMBER']) ? $rows['REQUEST_NUMBER'] : null;
            $languageId     = isset($rows['LANGUAGE_ID']) ? $rows['LANGUAGE_ID'] : null;
            $dbId           = isset($rows['DB_ID']) ? $rows['DB_ID'] : null;
            $patchLanguage  = isset($rows['PATCH_LANGUAGE']) ? $rows['PATCH_LANGUAGE'] : null;
            $mode           = isset($rows['MODE']) ? $rows['MODE'] : null;
            $versionStructure = isset($rows['VERSION_STRUCTURE']) ? $rows['VERSION_STRUCTURE'] : null;
            $this->downloadVersionService->requestDownloadVersion($requestType, $dbId, $email, $version,
                $initialPatch, $lastPatch, $productId, $languageId, $requestNumber, $patchLanguage, $mode,
                $versionStructure);
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }
} 